/*
 * Project name:
     Adc_Test (Demonstration on usage of the ADC library for dsPIC)
 * Copyright:
     (c) mikroElektronika, 2010.
 * Revision History:
     20091030:
       - Initial release;
 * Description:
     In this simple example, the analog input value is read on the channel 1,
     then the read value is converted to a string and sent through the UART1.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EASYPIC v7 for dsPIC30 - ac:ADC
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Close port RB10 at ADC INPUT, to enable ADC on channel 10 that is used in this example (board specific).
     - On EASYPIC v7 for dsPIC30 board, you can use the built-in potentiometer P3 to
       simulate analog input signal.
 */

unsigned int temp_res;

void main() {

  TRISB  = 0xFFFF;            // PORTB is input
  TRISD  = 0;                 // PORTD is output
  TRISF  = 0;                 // PORTF is output
  ADC1_Init();                // Enable ADC module

  do {
    temp_res = ADC1_Read(10);  // Get 10-bit results of AD conversion
    PORTD = temp_res;         // Send lower 8 bits to PORTD
    PORTF = temp_res >> 8;    // Send 2 most significant bits to RC1, RC0
  } while(1);
}