/*
 * Project name:
     I2C_Simple (Simple test of I2C library routines)
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20100511:
       - initial release;
 * Description:
     This program demonstrates usage of the I2C library routines. It
     establishes I2C bus communication with 24C02 EEPROM chip, writes one byte
     of data on some location, then reads it and displays it on PORTB.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EASYPIC v7 for dsPIC30 - ac:EEPROM
                      http://www.mikroe.com/store/components/various/
     Oscillator:      XT-PLL8, 80.000MHz
     Ext. Modules:    None.
                      http://www.mikroe.com/en/tools/eeprom/
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Pull-up I2C communication lines (RF2 and RF3) at I2C EEPROM
     - Turn off all PORT LEDs except PORTB at SW10.
 */
 
void main(){
  ADPCFG = 0xFFFF;           // initialize AN pins as digital

  LATB = 0;
  TRISB = 0;                 // Configure PORTB as output

  I2C1_Init(100000);         // initialize I2C communication
  I2C1_Start();              // issue I2C start signal
  I2C1_Write(0xA2);          // send byte via I2C  (device address + W)
  I2C1_Write(2);             // send byte (address of EEPROM location)
  I2C1_Write(0xF0);          // send data (data to be written)
  I2C1_Stop();               // issue I2C stop signal

  Delay_100ms();

  I2C1_Start();              // issue I2C start signal
  I2C1_Write(0xA2);          // send byte via I2C  (device address + W)
  I2C1_Write(2);             // send byte (data address)
  I2C1_Restart();            // issue I2C signal repeated start
  I2C1_Write(0xA3);          // send byte (device address + R)
  LATB = I2C1_Read(0u);      // Read the data (NO acknowledge)
  I2C1_Stop();               // issue I2C stop signal
}