/*
 * Project name:
     Seven Segment Display (The 'Hello World' example for the Seven Segment Display)
 * Copyright:
     (c) Mikroelektronika, 2014.
 * Revision History:
     20140220:
       - initial release (MC);
 * Description:
     This code demonstrates how to display number on one 7-segment display
     (common cathode). Display is connected to PORTB (RB0..RB7, segment A to
     RB0, segment B to RB1, etc); Number is incremented every 1s.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     dev.board:       easypic v7 for dspic30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
     - Turn on Seven Segment Display switches SW11.1, SW11.2, SW11.3 and SW11.4. (board specific)
 */

#include "Display_Utils.h"

unsigned short shifter, portd_index;
unsigned int   digit, number;
unsigned short portd_array[4];

//Timer1
//Prescaler 1:1; PR1 Preload = 20000; Actual Interrupt Time = 1 ms

//Place/Copy this part in declaration section
void InitTimer1(){
  T1CON	         = 0x8000;
  T1IE_bit	 = 1;
  T1IF_bit	 = 0;
  IPC0		 = IPC0 | 0x1000;
  PR1		 = 20000;
}

void Timer1Interrupt() iv IVT_ADDR_T1INTERRUPT{
  T1IF_bit	 = 0;
  LATC           = 0;                   // Turn off all 7seg displays
  LATF           = 0;
  LATB = portd_array[portd_index];      // bring appropriate value to PORTB

  if(shifter == 1)
    LATC13_bit = 1;
  if(shifter == 2)
    LATC14_bit = 1;
  if(shifter == 4)
    LATF4_bit = 1;
  if(shifter == 8)
    LATF5_bit = 1;

  // move shifter to next digit
  shifter <<= 1;
  if (shifter > 8u)
    shifter = 1;

  // increment portd_index
  portd_index++ ;
  if (portd_index > 3u)
    portd_index = 0;             // turn on 1st, turn off 2nd 7seg.
}

void main() {
  ADPCFG = 0xFFFF;               // initialize AN pins as digital

  TRISB = 0;                     // Configure PORTA as output
  TRISC = 0;
  TRISF = 0;
  LATB  = 0;                     // Clear PORTA
  LATC  = 0;
  LATF  = 0;

  InitTimer1();                  // Initialize timer 1

  digit = 0;
  portd_index = 0;
  shifter = 1;

  number = 1234;                 // Initial number value

  do {
    digit = number / 1000u ;             // extract thousands digit
    portd_array[3] = mask(digit);        // and store it to PORTD array
    digit = (number / 100u) % 10u;       // extract hundreds digit
    portd_array[2] = mask(digit);        // and store it to PORTD array
    digit = (number / 10u) % 10u;        // extract tens digit
    portd_array[1] = mask(digit);        // and store it to PORTD array
    digit = number % 10u;                // extract ones digit
    portd_array[0] = mask(digit);        // and store it to PORTD array

    Delay_ms(1000);                      // one second delay

    number++ ;                           // increment number
    if (number > 9999u)
      number = 0;

  } while(1);                            // endless loop
}