/*
 * Project name:
     Piezo buzzer keypad example (Demonstration of usage of the Piezzo Buzzer)
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20100512:
       - initial release;
 * Description:
      A simple example of using Piezo Buzzer.
      By pressing the buttons on PORTB, buzzer plays a note
      by generating a PWM signal of appropriate frequency with 1/2 duty.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EASYPIC v7 for dsPIC30 - ac:Sound
                      http://www.mikroe.com/store/components/various/
     Oscillator:      XT-PLL8, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - At Button Press Level set SW7.1 (PB)
     - On J12 set jumper to RA11 (Piezo Buzzer)
     - Pull down RB7... RB3 at PORTB
 */

void Tone1() {
  Sound_Play(659, 250);   // Frequency = 659Hz, duration = 250ms
}

void Tone2() {
  Sound_Play(698, 250);   // Frequency = 698Hz, duration = 250ms
}

void Tone3() {
  Sound_Play(784, 250);   // Frequency = 784Hz, duration = 250ms
}

void Melody() {           // Plays the melody "Yellow house"
  Tone1(); Tone2(); Tone3(); Tone3();
  Tone1(); Tone2(); Tone3(); Tone3();
  Tone1(); Tone2(); Tone3();
  Tone1(); Tone2(); Tone3(); Tone3();
  Tone1(); Tone2(); Tone3();
  Tone3(); Tone3(); Tone2(); Tone2(); Tone1();
}

void ToneA() {
  Sound_Play( 880, 50);
}
void ToneC() {
  Sound_Play(1046, 50);
}
void ToneE() {
  Sound_Play(1318, 50);
}

void Melody2() {
  unsigned short i;
  for (i = 9; i > 0; i--) {
    ToneA(); ToneC(); ToneE();
  }
}

void main() {

  ADPCFG = 0xFFFF;                  // Configure AN pins as digital
  TRISB = 0xF8;                     // Configure RB7..RB3 as input
  LATB = 0;

  Sound_Init(&PORTA, 11);
  Sound_Play(880, 1000);            // Play sound at 880Hz for 1 second

  while (1) {
    if (Button(&PORTB,7,1,1))       // RB7 plays Tone1
      Tone1();
    while (RB7_bit);                // Wait for button to be released

    if (Button(&PORTB,6,1,1))       // RB6 plays Tone2
      Tone2();
    while (RB6_bit);                // Wait for button to be released

    if (Button(&PORTB,5,1,1))       // RB5 plays Tone3
      Tone3();
    while (RB5_bit);                // Wait for button to be released

    if (Button(&PORTB,4,1,1))       // RB4 plays Melody2
      Melody2();
    while (RB4_bit);                // Wait for button to be released

    if (Button(&PORTB,3,1,1))       // RB3 plays Melody
      Melody();
    while (RB4_bit);                // Wait for button to be released
  }
}