/*
 * Project name:
     I2C_Simple (Simple test of I2C library routines)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Description:
     This program demonstrates usage of the I2C library routines. It
     establishes I2C bus communication with 24C02 EEPROM chip, writes one byte
     of data on some location, then reads it and displays it on PORTB.
 * Test configuration:
     MCU:             P24EP512GU810
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70616F.pdf
     Dev.Board:       EasyPIC Fusion v7 -  ac:Serial_EEPROM
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT, 8.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn on I2C lines at SW14.3 (RA2) and SW14.4 (RA3). (board specific)
     - Turn off PORTA LEDs at SW15.1. (board specific)
     - Turn on PORTB and PORTD LEDs SW15.3 and SW15.5. (board specific)
 */

unsigned short i;
char j;
 
void EEPROM_24C02_Init() {
  I2C2_Init(100000);
}

//--------------- Writes data to 24C02 EEPROM - signle location
void EEPROM_24C02_WrSingle(unsigned short wAddr, unsigned short wData) {
  I2C2_Start();              // issue I2C start signal
  I2C2_Write(0xA0);          // send byte via I2C  (command to 24cO2)
  I2C2_Write(wAddr);         // send byte (address of EEPROM location)
  I2C2_Write(wData);         // send data (data to be written)
  I2C2_Stop();
}

//--------------- Reads data from 24C02 EEPROM - single location (random)
unsigned short EEPROM_24C02_RdSingle(unsigned short rAddr) {
  unsigned short reslt;

  I2C2_Start();              // issue I2C start signal
  I2C2_Write(0xA0);          // send byte via I2C  (device address + W)
  I2C2_Write(rAddr);         // send byte (data address)
  I2C2_Restart();            // issue I2C signal repeated start
  I2C2_Write(0xA1);          // send byte (device address + R)
  reslt = I2C2_Read(1);      // Read the data (NO acknowledge)
  while (!I2C2_Is_Idle())
    asm nop;                 // Wait for the read cycle to finish
  I2C2_Stop();
  return reslt;
}

void main(){
  // Setting output frequency to 140MHz
  PLLFBD = 70;             // PLL multiplier M=70
  CLKDIV = 0x0000;         // PLL prescaler N1=2, PLL postscaler N2=2

  ANSELA = 0x00;           // Convert all I/O pins to digital
  ANSELB = 0x00;
  ANSELC = 0x00;
  ANSELD = 0x00;
  ANSELE = 0x00;
  ANSELG = 0x00;

  TRISB = 0;               // Configure PORTB as output
  TRISD = 0;               // Configure PORTB as output
  LATB = 0;                // Set PORTB value to zero
  LATD = 0;                // Set PORTD value to zero

  EEPROM_24C02_Init();     // performs I2C initialization
  j = 0x00;
  for(i = 0x00; i < 0x80; i++) {
    EEPROM_24C02_WrSingle(i,j);
    j++;
    delay_ms(5);
  }
  for(i = 0x00; i < 0x80; i++){
    LATD = i;
    LATB = EEPROM_24C02_RdSingle(i);
    delay_ms(100);
  }
}