/*
 * Project name:
     GLCD_Test (Demonstration of the GLCD library routines)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Description:
     This is a simple demonstration of the GLCD library routines:
     - Init and Clear (pattern fill)
     - Image display
     - Basic geometry - lines, circles, boxes and rectangles
     - Text display and handling
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    GLCD 128x64, KS108/107 controller ac:GLCD
                      http://www.mikroe.com/store/components/various/
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * Notes:
     - Turn on GLCD backlight switch SW11.1.
*/

//Declarations------------------------------------------------------------------
const code char truck_bmp[1024];
//--------------------------------------------------------------end-declarations

// Glcd module connections
sbit GLCD_D0 at RE0_bit;
sbit GLCD_D1 at RE1_bit;
sbit GLCD_D2 at RE2_bit;
sbit GLCD_D3 at RE3_bit;
sbit GLCD_D4 at RE4_bit;
sbit GLCD_D5 at RE5_bit;
sbit GLCD_D6 at RE6_bit;
sbit GLCD_D7 at RE7_bit;

sbit GLCD_D0_Direction at TRISE0_bit;
sbit GLCD_D1_Direction at TRISE1_bit;
sbit GLCD_D2_Direction at TRISE2_bit;
sbit GLCD_D3_Direction at TRISE3_bit;
sbit GLCD_D4_Direction at TRISE4_bit;
sbit GLCD_D5_Direction at TRISE5_bit;
sbit GLCD_D6_Direction at TRISE6_bit;
sbit GLCD_D7_Direction at TRISE7_bit;

sbit GLCD_CS1 at LATD5_bit;
sbit GLCD_CS2 at LATD4_bit;
sbit GLCD_RS  at LATD9_bit;
sbit GLCD_RW  at LATG6_bit;
sbit GLCD_EN  at LATD10_bit;
sbit GLCD_RST at LATD7_bit;

sbit GLCD_CS1_Direction at TRISD5_bit;
sbit GLCD_CS2_Direction at TRISD4_bit;
sbit GLCD_RS_Direction  at TRISD9_bit;
sbit GLCD_RW_Direction  at TRISG6_bit;
sbit GLCD_EN_Direction  at TRISD10_bit;
sbit GLCD_RST_Direction at TRISD7_bit;
// End Glcd module connections

void delay2S(){                                  // 2 seconds delay function
  Delay_ms(2000);
}

void main() {
  unsigned short ii;
  char *someText;

  // PLL settings
  CLKDIVbits.PLLPRE = 0; // PLLPRE<4:0> = 0 -> N1 = 2 8MHz / 2 = 4MHz
  // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD = 38; // PLLDIV<8:0> = 38 -> M = 40 4MHz * 40 = 160MHz
  // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0; // PLLPOST<1:0> = 0 -> N2 = 2 160MHz / 2 = 80MHz
  // (must be within 12.5 MHz to 80 MHz range)

  AD1PCFGL = 0xFFFF; // Set All pins as digital
  AD1PCFGH = 0xFFFF;

  Glcd_Init();                                   // Initialize GLCD
  Glcd_Fill(0x00);                               // Clear GLCD

  while(1) {
    Glcd_Image(truck_bmp);                       // Draw image
    delay2S(); delay2S();

    Glcd_Fill(0x00);                             // Clear GLCD

    Glcd_PartialImage(0,0,68,30,128,64,truck_bmp); // Partial image
    Delay_ms(500);
    Glcd_PartialImage(24,16,68,30,128,64,truck_bmp);
    Delay_ms(500);
    Glcd_PartialImage(56,34,68,30,128,64,truck_bmp);

    Delay2s(); Delay2s();
    Glcd_Fill(0x00);                             // Clear GLCD

    Glcd_Box(62,40,124,56,1);                       // Draw box
    Glcd_Rectangle(5,5,84,35,1);                    // Draw rectangle
    Delay_ms(1000);
    Glcd_Rectangle_Round_Edges(2,2,87,38,7,1);
    Delay_ms(1000);
    Glcd_Rectangle_Round_Edges_Fill(8,8,81,32,12,1);
    Delay_ms(1000);
    Glcd_Line(0, 0, 127, 63, 1);                    // Draw line
    Delay2s();

    for(ii = 5; ii < 60; ii+=5 ){                // Draw horizontal and vertical lines
      Delay_ms(250);
      Glcd_V_Line(2, 54, ii, 1);
      Glcd_H_Line(2, 120, ii, 1);
    }

    delay2S();

    Glcd_Fill(0x00);                                       // Clear GLCD
    Glcd_Set_Font(Font_Glcd_Character8x7, 8, 7, 32);       // Choose font, see __Lib_GLCDFonts.c in Uses folder
    Glcd_Write_Text("mikroE", 1, 7, 2);                    // Write string

    for (ii = 1; ii <= 10; ii++)                           // Draw circles
      Glcd_Circle(63,32, 3*ii, 1);
    delay2S();

      Glcd_Circle_Fill(63,32, 30, 1);                 // Draw circles
      Delay2S();

      Glcd_Box(10,20, 70,63, 2);                      // Draw box
      Delay2s();

      Glcd_Fill(0xFF);                                // Fill GLCD

      Glcd_Set_Font(Character8x7, 8, 7, 32);         // Change font
      someText = "8x7 Font";
      Glcd_Write_Text(someText, 5, 0, 2);             // Write string
      Delay2s();

      Glcd_Set_Font(System3x5, 3, 5, 32);            // Change font
      someText = "3X5 CAPITALS ONLY";
      Glcd_Write_Text(someText, 60, 2, 2);            // Write string
      Delay2s();

      Glcd_Set_Font(font5x7, 5, 7, 32);              // Change font
      someText = "5x7 Font";
      Glcd_Write_Text(someText, 5, 4, 2);             // Write string
      Delay2s();

      Glcd_Set_Font(FontSystem5x7_v2, 5, 7, 32);     // Change font
      someText = "5x7 Font (v2)";
      Glcd_Write_Text(someText, 50, 6, 2);            // Write string
      Delay2s();
  }
}