/* 
 * Project name:
     LM35 (analog on-board temperature sensor)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Description:
     This example utilizes temperature sensor on EasyPIC Fusion V7 board. Temperature (in form of
     ADC value) is read from temperature pin RB0/AN0 and then converted, using exact formula,
     to Celsius degrees format. Value is then displayed on TFT display.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz - ac:LM35
     Ext. Modules:    EasyTFT display - ac:EasyTFT
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn on LM35 switch at SW11.4. (board specific)
     - Turn on TFT backlight switch SW11.1. (board specific)
     - Turn off PORTB LEDs at SW15. (board specific)
 */

#include "resources.h"

const unsigned long VREF = 3300;   // Voltage reference in mV
char txt[10];// = "000.0   C";

void Init() {
  // PLL settings
  CLKDIVbits.PLLPRE = 0;      // PLLPRE<4:0> = 0  ->  N1 = 2    8MHz / 2 = 4MHz
                              // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD =   38;              // PLLDIV<8:0> = 38 ->  M = 40    4MHz * 40 = 160MHz
                              // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0;     // PLLPOST<1:0> = 0 ->  N2 = 2    160MHz / 2 = 80MHz
                              // (must be within 12.5 MHz to 80 MHz range)

  AD1PCFGL = 0xFFFE;          // Set PCFG0 as analog input
  AD1PCFGH = 0xFFFF;          // Ans all other pins as digital
  TRISB0_bit = 1;             // Set RB0 as input
  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320, 240);         // Initialize TFT display
  TFT_BLED = 1;               // Turn on TFT backlight
  TRISB0_bit = 1;             // Set PORTB.B0 as input
}

void DrawFrame(){
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("LM35  Example", 80, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}

void Display_Temperature(unsigned long temp_mV) {
  ByteToStr(temp_mV/10, txt);
  txt[3] = '.';
  txt[4] = temp_mV % 10 + 48;
  txt[5] = ' ';
  txt[6] = ' ';
  txt[7] = 176;
  txt[8] = 'C';
  txt[9] = 0;


  // Print temperature on TFT
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Pen(CL_WHITE,1);
  TFT_Rectangle(85,100,200,140);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text(txt, 85, 100);
}

void main() {
  unsigned long temp;

  Init();
  DrawFrame();

  TFT_Set_Font(&tahoma29x29_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Temperature:  ", 75, 65);

  ADC1_Init();                 // Initialize ADC

  do {
    temp = ADC1_Get_Sample(0);
    temp = (temp * VREF) / 1024;
    Display_Temperature(temp);
    Delay_ms(1000);
  } while(1);
}