/*
 * Project name:
     Led_Curtain (The simplest simple example)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Description:
     Simple "Hello world" example for the world of dsPIC MCUs;
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7 - ac:LED
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.0000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn ON PORTA, B, C, D, F & G LEDS at SW15. (board specific)
 */

char counter;

void wait() {
  Delay_ms(100);
}

void main() {

  // PLL settings
  CLKDIVbits.PLLPRE = 0;   // PLLPRE<4:0> = 0  ->  N1 = 2    8MHz / 2 = 4MHz
                           // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD =   38;           // PLLDIV<8:0> = 38 ->  M = 40    4MHz * 40 = 160MHz
                           // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0;  // PLLPOST<1:0> = 0 ->  N2 = 2    160MHz / 2 = 80MHz
                           // (must be within 12.5 MHz to 80 MHz range)

  AD1PCFGL = 0xFFFE;       // Set All pins as digital
  AD1PCFGH = 0xFFFF;

  TRISA = 0;          // set direction to be output
  TRISB = 0;          // set direction to be output
  TRISC = 0;          // set direction to be output
  TRISD = 0;          // set direction to be output
  TRISE = 0;          // set direction to be output
  TRISF = 0;          // set direction to be output
  TRISG = 0;          // set direction to be output

  LATA = 0;           // turn OFF the LATA leds
  LATB = 0;           // turn OFF the LATB leds
  LATC = 0;           // turn OFF the LATC leds
  LATD = 0;           // turn OFF the LATD leds
  LATE = 0;           // turn OFF the LATE leds
  LATF = 0;           // turn OFF the LATF leds
  LATG = 0;           // turn OFF the LATG leds

  counter = 0;
  while(1) {
    // TURN ON LEDs
    for (counter = 0; counter < 4; counter++){
      LATA |= 0x8888 >> counter;
      LATB |= 0x8888 >> counter;
      LATC |= 0x8888 >> counter;
      LATD |= 0x8888 >> counter;
      LATE |= 0x8888 >> counter;
      LATF |= 0x8888 >> counter;
      LATG |= 0x8888 >> counter;
      Delay_ms(100);
    }
    // TURN OFF LEDs
    for (counter = 0; counter < 4; counter++){
      LATA &= 0x7777 >> counter;
      LATB &= 0x7777 >> counter;
      LATC &= 0x7777 >> counter;
      LATD &= 0x7777 >> counter;
      LATE &= 0x7777 >> counter;
      LATF &= 0x7777 >> counter;
      LATG &= 0x7777 >> counter;
      Delay_ms(100);
    }
  }
}