/*
 * Project name:
     IirFilter
 * Copyright:
     (c) MikroElektronika, 2012.
 * Description:
     This code was generated by filter designer tool by mikroElektronika
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    mE ac:DAC_SPI
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:

 Filter setup:
     Filter kind: IIR
     Filter type: Lowpass filter
     Filter order: 3
     Design method: Butterworth
     Filter borders:
     Wpass:4000 Hz
     
     Sampling Frequency: 40 kHz
*/


const unsigned int  BUFFER_SIZE  = 8;
const unsigned int  FILTER_ORDER  = 3;
const signed int    COEFF_B[FILTER_ORDER+1] = {0x2511, 0x6F33, 0x6F33, 0x2511};
const signed int    COEFF_A[FILTER_ORDER+1] = {0x4000, 0x8F5B, 0x4BB5, 0xEE34};
const unsigned int  SCALE_B       = 4;
const unsigned int  SCALE_A       = -1;

const unsigned int  CS_PIN        = 1;   // DAC CS pin

unsigned int inext;                      // Input buffer index
ydata unsigned int input[BUFFER_SIZE];   // Input buffer
ydata unsigned int output[BUFFER_SIZE];  // Output buffer

// This is ADC interrupt handler.
// Analog input is sampled and the value is stored into input buffer.
// Input buffer is then passed through filter.
// Finally, the resulting output sample is sent to DAC.

void ADC1Int() iv IVT_ADDR_ADC1INTERRUPT {                // ADC interrupt handler
unsigned int CurrentValue;
  latd  = portd ^ 0xffff;
  input[inext] = ADCBUF0;                // Fetch sample

  CurrentValue = IIR_Radix( SCALE_B,     //
                            SCALE_A,     //
                            COEFF_B,     // b coefficients of the filter
                            COEFF_A,     // a coefficients of the filter
                            FILTER_ORDER+1,// Filter order + 1
                            input,       // Input buffer
                            BUFFER_SIZE, // Input buffer length
                            output,      // Input buffer
                            inext);      // Current sample

//  CurrentValue = 2048;
  output[inext] = CurrentValue;

  inext = (inext+1) & (BUFFER_SIZE-1);   // inext = (inext + 1) mod BUFFER_SIZE;

  LATG.CS_PIN = 0;                       // CS enable for DAC
  SPI2_Write(0x3000 | CurrentValue);     // Write CurrentValue to DAC (0x3 is required by DAC)
  LATG.CS_PIN    = 1;                    // CS disable for DAC

  IFS0.F13 = 0;                          // Clear AD1IF
} //~

// This is Timer1 interrupt handler.
// It is used to start ADC at
//     periodic intervals.
void Timer1Int() iv IVT_ADDR_T1INTERRUPT {              // Timer1 interrupt handler
  if (AD1CON1.DONE == 1){                 // If ADC is not busy
    AD1CON1.SAMP  = 1;                    // Start new sample
  }

  LATD = PORTD ^ 0xFFFF;
  IFS0.F3    = 0;                        // Clear TMR1IF
} //~


// Main program starts here.
// Firstly, hardware peripherals are initialized and then
//   the program goes to an infinite loop, waiting for interrupts.
void main() {

  // PLL settings
  CLKDIVbits.PLLPRE = 0; // PLLPRE<4:0> = 0 -> N1 = 2 8MHz / 2 = 4MHz
  // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD = 38; // PLLDIV<8:0> = 38 -> M = 40 4MHz * 40 = 160MHz
  // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0; // PLLPOST<1:0> = 0 -> N2 = 2 160MHz / 2 = 80MHz
  // (must be within 12.5 MHz to 80 MHz range)

  TRISD = 0;
  // DAC setup
  TRISG.CS_PIN   = 0;                    // CS pin
  LATG.CS_PIN    = 1;                    // Set CS to inactive

  // SPI setup
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_16_BIT, _SPI_PRESCALE_SEC_2, _SPI_PRESCALE_PRI_16, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);

  inext   = 0;                           // Initialize buffer index
  Vector_Set(input, BUFFER_SIZE, 0);     // Clear input buffer
  Vector_Set(output, BUFFER_SIZE, 0);    // Clear output buffer
  
  TRISB   = 0xFFFF;                      // Use PORTB for input signal
  AD1CON1 = 0x00E2;                      // Auto-stop sampling, unsigned integer out
  AD1CON2 = 0x0000;
  AD1CON3 = 0x021A;                      // Sampling time= 3*Tad, minimum Tad selected
  ADPCFG  = 0x0000;                      // Configure PORTB as ADC input port
  AD1CHS0 = 0x000A;                      // Sample input on RB10
  AD1CSSL = 0;                           // No input scan
  ADPCFG  = 0x0000;                      // Configure PORTB as analog ADC input port

  // Interrupts setup
  IFS0    = 0;
  IFS1    = 0;
  IFS2    = 0;
  INTCON1 = 0x8000;                       // Nested interrupts DISABLED
  INTCON2 = 0;
  IEC0    = 0x2008;                       // Timer1 and ADC interrupts ENABLED
  IPC0.F12= 1;                            // Timer1 interrupt priority level = 1
  IPC3.F4 = 1;                            // ADC interrupt priority level = 1

  // Timer1 setup
  PR1     = 250;                          // Sampling ~= 40 kHz. The value of 250 is calculated for 80MHz clock.
  T1CON   = 0x8000;                       // Timer1 ON, internal clock FCY, prescaler 1:1
  AD1CON1.F15 = 1;
  AD1CON1.SAMP  = 1;
  while (1);                              // Infinite loop,
                                          //   wait for interrupts
} //~