/*
 * Project name:
     Matrix_Test (Matrix library test example)
 * Platforms Supported:
     dsPIC
 * Copyright:
     (c) MikroElektronika, 2009.
 * Revision History:
     20091030:
       - Initial release
 * Description:
     This is a simple demonstration of Matrix library functions. The examples
     should be self-explanatory, for more information please take a look at
     mikroC's Help. To familiarise yourself more with matrix operation you can go to
     http://en.wikipedia.org/wiki/Matrix_%28mathematics%29
     This example is meant to be tested on the Debugger Tool, but it can
     be easily modified to be tested on the "real" hardware.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     None.
 */

int mx1[6] = {1,2,3,
              4,5,6};
int mx2[6] = {2,2,2,
              2,2,2};
int mx3[6] = {0x4000,0x4000,0x4000,           // 0.5
              0x4000,0x4000,0x4000};
int mx4[6] = {0xA000,0xC000,0xE000,           // -0.75 -0.5 -0.25
              0x2000,0x4000,0x6000};          //  0.25  0.5  0.75
                            
int mxDest[9];

void main() {

  // PLL settings
  CLKDIVbits.PLLPRE = 0; // PLLPRE<4:0> = 0 -> N1 = 2 8MHz / 2 = 4MHz
  // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD = 38; // PLLDIV<8:0> = 38 -> M = 40 4MHz * 40 = 160MHz
  // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0; // PLLPOST<1:0> = 0 -> N2 = 2 160MHz / 2 = 80MHz
  // (must be within 12.5 MHz to 80 MHz range)

  Matrix_Add(mx1,mx2, mxDest,2,3);            // {3, 4, 5, 6, 7, 8, 0, 0, 0}
  Matrix_Subtract(mxDest, mx2, mxDest,2,3);   // {1, 2, 3, 4, 5, 6, 0, 0, 0}
  Matrix_Transpose(mx1, mxDest, 2,3);         // {1, 4, 2, 5, 3, 6, 0, 0, 0}
  
  Matrix_Scale(0x4000, mx4, mxDest, 2,3);     // {0xD000, 0xE000, 0xF000, 0x1000, 0x2000, 0x3000, 0x0000, 0x0000, 0x0000}
                                              //  -0.375,  -0.25, -0.125,  0.125,   0.25,  0.375
                                              
  Matrix_Multiply(mx3,mx3,mxDest,2,2,3);      // {0x6000, 0x6000, 0x6000, 0x6000, 0x2000, 0x3000, 0x0000, 0x0000, 0x0000}
                                              //    0.75,   0.75,   0.75,   0.75
}