/*
 * Project name:
     Q15_Demo (Q15 library test example)
 * Copyright:
     (c) MikroElektronika, 2011.
 * Description:
     This is a simple demonstration on Q15 library functions.
     The examples should be self-explanatory, for more information
     please take a look at Help.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - None.
*/

int   Q15_1, Q15_2;
int   i;
float f;
char  stringQ15[19], status;

void main() {

  // PLL settings
  CLKDIVbits.PLLPRE = 0; // PLLPRE<4:0> = 0 -> N1 = 2 8MHz / 2 = 4MHz
  // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD = 38; // PLLDIV<8:0> = 38 -> M = 40 4MHz * 40 = 160MHz
  // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0; // PLLPOST<1:0> = 0 -> N2 = 2 160MHz / 2 = 80MHz
  // (must be within 12.5 MHz to 80 MHz range)

  // Q15_Ftoi...................................................................
  f = 0.5;
  Q15_1 = Q15_Ftoi(f);                                        // Q15_1 = 0x4000;
  f = 1.5;
  Q15_1 = Q15_Ftoi(f);                                        // Q15_1 = 0x7FFF;

  // Q15_Itof...................................................................
  Q15_1 = 0x1000;
  f = Q15_Itof(Q15_1);                                        // f = 0.125;

  // Q15_Itoa...................................................................
  Q15_1 = 0xFFFF;
  Q15_Itoa(Q15_1, stringQ15);                                 // stgingQ15 = "-0.000030517578125"
  Q15_1 = 0x3000;
  Q15_Itoa(Q15_1, stringQ15);                                 // stgingQ15 = " 0.375"
  Q15_1 = 0x4000;
  Q15_Itoa(Q15_1, stringQ15);                                 // stgingQ15 = " 0.5"

  // Q15_Atoi...................................................................
  status = Q15_Atoi("+0.25", &Q15_1);                         // Q15_1 = 0x2000; status = 0
  status = Q15_Atoi("        +0.7", &Q15_1);                  // Q15_1 = 0x5999; status = 0
  status = Q15_Atoi("mikroe.25", &Q15_1);                     // status = 2
  status = Q15_Atoi("2.25", &Q15_1);                          // status = 1
  status = Q15_Atoi("-1", &Q15_1);                            // Q15_1 = 0x8000; status = 0
  status = Q15_Atoi("0.9999999999", &Q15_1);                  // Q15_1 = 0x7fff; status = 0

  // Q15_Abs....................................................................
  Q15_1 = Q15_Abs(0x8000);                      // Q15_1 = 0x7FFF
  Q15_1 = Q15_Abs(0xE000);                      // Q15_1 = 0X2000




  // Q15_Add....................................................................
  status = Q15_Add(0x7000, 0x4000, &Q15_1);          // Q15_1 = 0x7FFF; status = 1
  status = Q15_Add(0x2000, 0x4000, &Q15_1);          // Q15_1 = 0x6000; status = 0
  status = Q15_Add(0x6000, 0x5000, &Q15_1);          // Q15_1 = 0x7FFF; status = 1
  status = Q15_Add(0xC000, 0xC000, &Q15_1);          // Q15_1 = 0x8000; status = 0

  // Q15_Sub....................................................................
  status = Q15_Sub(0x2000, 0x4000, &Q15_1);          // Q15_1 = 0xE000; status = 0
  status = Q15_Sub(0x7000, 0xC000, &Q15_1);          // Q15_1 = 0x7FFF; status = 1
  status = Q15_Sub(0xC000, 0x7000, &Q15_1);          // Q15_1 = 0x8000; status = 1





  // Q15_Cos....................................................................
  Q15_1 = Q15_Ftoi(0.5);
  Q15_2 = Q15_Cos (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = cos(0.5) = 0.877

  Q15_1 = Q15_Ftoi(1);
  Q15_2 = Q15_Cos (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = cos(1) = 0.540

  // Q15_Sin....................................................................
  Q15_1 = Q15_Ftoi(0.5);
  Q15_2 = Q15_Sin (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = sin(0.5) = 0.479

  Q15_1 = Q15_Ftoi(1);
  Q15_2 = Q15_Sin (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = sin(1) = 0.841

  // Q15_Tan....................................................................
  Q15_1 = Q15_Ftoi(0.5);
  Q15_2 = Q15_Tan (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = tan(0.5) = 0.546

  Q15_1 = Q15_Ftoi(0.78546);
  Q15_2 = Q15_Tan (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = tan(0.78546) = 0.999

  // Q15_Asin...................................................................
  Q15_1 = Q15_Ftoi(0.5);
  Q15_2 = Q15_Asin (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = asin(0.5) = 0.523

  Q15_1 = Q15_Ftoi(0.8416137);
  Q15_2 = Q15_Asin (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = asin(0.8416137) = 0.999

  // Q15_Acos...................................................................
  Q15_1 = Q15_Ftoi(0.9);
  Q15_2 = Q15_Acos (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = acos(0.9) = 0.451

  Q15_1 = Q15_Ftoi(0.8);
  Q15_2 = Q15_Acos (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = acos(0.8) = 0.643

  // Q15_Atan...................................................................
  Q15_1 = Q15_Ftoi(0.5);
  Q15_2 = Q15_Atan (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = atan(0.5) = 0.464

  Q15_1 = Q15_Ftoi(1);
  Q15_2 = Q15_Atan (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = atan(1) = 0.791

  // Q15_Log....................................................................
  Q15_1 = Q15_Ftoi(0.5);
  Q15_2 = Q15_Log (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = log(0.5) = -0.301

  Q15_1 = Q15_Ftoi(1);
  Q15_2 = Q15_Log (Q15_1);
  f = Q15_Itof(Q15_2);                          // f = log(1) = 0

  //////////////////////////////////////////////////////////////////////////////
}