/*
 * Project name:
     Vectors_Test (Vectors library test example)
 * Copyright:
     (c) MikroElektronika, 2009.
 * Revision History:
     20091030:
       - Initial release
 * Description:
     This is a simple demonstration on Vector library functions.
     http://en.wikipedia.org/wiki/Vector_%28mathematics_and_physics%29
     The examples should be self-explanatory, for more information
     please take a look at mikroC's Help. The example is meant to be
     tested on the Debugger Tool, although it can
     be easily modified to be tested on the "real" hardware.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - None.
*/

unsigned vec1[3] = {1,2,3};
unsigned vec2[3] = {1,1,1};
unsigned vecDest[3];
unsigned vConDest[10];
unsigned vConDest2[10];
unsigned index;
unsigned rslt;

void main() {

  // PLL settings
  CLKDIVbits.PLLPRE = 0; // PLLPRE<4:0> = 0 -> N1 = 2 8MHz / 2 = 4MHz
  // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD = 38; // PLLDIV<8:0> = 38 -> M = 40 4MHz * 40 = 160MHz
  // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0; // PLLPOST<1:0> = 0 -> N2 = 2 160MHz / 2 = 80MHz
  // (must be within 12.5 MHz to 80 MHz range)

  Vector_Add(vecDest, vec1, vec2, 3);            // 2,3,4
  Vector_Subtract(vecDest, vecDest, vec2, 3);    // 1,2,3
  Vector_Negate(vecDest, vecDest, 3);            // -1,-2,-3

  Vector_Multiply(vec1, vec2, vConDest, 3);      // 0,0,0

  rslt = Vector_Max(vecDest, 3, &index);         // -1
  rslt = Vector_Min(vecDest, 3, &index);         // -3

  Vector_Set(vec2, 3, 0x4000);                   // {0x4000, 0x4000, 0x4000}

  Vector_Convolve(vec2,vec2,vConDest2,3,3);      // vConDest2 {0x2000,0x4000,0x6000,0x4000,0x2000,0x0000...
  Vector_Correlate(vec2,vec2,vConDest,3,3);      // vConDest  {0x2000,0x4000,0x6000,0x4000,0x2000,0x0000...

  rslt = Vector_Dot(vec2,vec2,3);                // 0x6000

}