/*
 * Project name:
     I2C_Simple (Simple test of I2C library routines)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091030:
       - initial release;
 * Description:
     This program demonstrates usage of the I2C library routines. It
     establishes I2C bus communication with 24C02 EEPROM chip, writes one byte
     of data on some location, then reads it and displays it on PORTB.
 * Test configuration:
     MCU:             PICF4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41291F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL, 8.0000 MHz
     Ext. Modules:    mE ac:EEPROM (24C02)
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Use SmartADAPT to match the I2C pinout of the dsPIC30F4013 and EEPROM module. (board specific)
     - Pull-up I2C communication lines (RF2 and RF3)
       place jumper J5 in upper position. (board specific)
     - Turn off all PORT LEDs except PORTB at SW10. (board specific)
 */
 
void main(){

  ADPCFG = 0xFFFF;            // initialize AN pins as digital

  LATB = 0;                   // Set PORTB value to zero
  TRISB = 0;                  // Configure PORTB as output

  I2C1_Init(100000);          // initialize I2C communication
  I2C1_Start();               // issue I2C start signal
  I2C1_Write(0xA2);           // send byte via I2C  (device address + W)
  I2C1_Write(2);              // send byte (address of EEPROM location)
  I2C1_Write(0xAA);           // send data (data to be written)
  I2C1_Stop();                // issue I2C stop signal

  Delay_100ms();

  I2C1_Start();               // issue I2C start signal
  I2C1_Write(0xA2);           // send byte via I2C  (device address + W)
  I2C1_Write(2);              // send byte (data address)
  I2C1_Restart();             // issue I2C signal repeated start
  I2C1_Write(0xA3);           // send byte (device address + R)
  LATB = I2C1_Read(1);        // Read the data (NO acknowledge)
  I2C1_Stop();                // issue I2C stop signal
}