/*
 * Project name:
     Keypad_Test (Demonstration of usage of the Keypad Library)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091030:
       - initial release;
 * Description:
      A simple example of using the keypad library for mikroC PRO for dsPIC30/33 and PIC24.
      It supports keypads with 1..4 rows and 1..4 columns. The code being
      returned by the Keypad_Key_Press() and Keypad_Key_Click() functions is
      in range from 1..16. In this example, the code returned by the keypad
      library functions is transformed into ASCII codes [0..9,A..F] and sent to UART.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL 8x, 10.00000 MHz
     Ext. Modules:    ac:Keypad_4x4 module at PORTB
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Pull-down PORTB. (board specific)
     - Turn on RF3 (TX) at SW9 and RF2 (RX) at SW8. (board specific)
 */

unsigned short kp, oldstate = 0;
char txt[6];

// Keypad module connections
unsigned keypadPort at PORTB;
unsigned keypadPort_Direction at TRISB;
// End Keypad module connections

void main() {
  ADPCFG = 0xFFFF;
  UART1_Init(9600);
  Delay_ms(100);
  Keypad_Init();                           // Initialize Keypad

  UART1_Write_Text("Press any key on your keypad...");
  UART1_Write(10);
  UART1_Write(13);

  do {
    kp = 0;                                // Reset key code variable

    // Wait for key to be pressed and released
    do
      // kp = Keypad_Key_Press();          // Store key code in kp variable
      kp = Keypad_Key_Click();             // Store key code in kp variable
    while (!kp);
   // Prepare value for output, transform key to it's ASCII value
    switch (kp) {
      //case 10: kp = 42; break;  // '*'   // Uncomment this block for keypad4x3
      //case 11: kp = 48; break;  // '0'
      //case 12: kp = 35; break;  // '#'
      //default: kp += 48;

      case  1: kp = 49; break; // 1        // Uncomment this block for keypad4x4
      case  2: kp = 50; break; // 2
      case  3: kp = 51; break; // 3
      case  4: kp = 65; break; // A
      case  5: kp = 52; break; // 4
      case  6: kp = 53; break; // 5
      case  7: kp = 54; break; // 6
      case  8: kp = 66; break; // B
      case  9: kp = 55; break; // 7
      case 10: kp = 56; break; // 8
      case 11: kp = 57; break; // 9
      case 12: kp = 67; break; // C
      case 13: kp = 42; break; // *
      case 14: kp = 48; break; // 0
      case 15: kp = 35; break; // #
      case 16: kp = 68; break; // D

    }
    UART1_Write_Text("Key pressed: ");
    UART1_Write(kp);                       // Send value of pressed button to UART
    UART1_Write(10);
    UART1_Write(13);
  } while (1);
}