/*
 * Project name:
     PortExpander (Demonstration of the Port Expander library routines)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - initial release;
 * Description:
     This project is simple demonstration how to use Port Expander Library functions.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    mE ac:Port_Expander board on PORTF
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn on LEDs on PORTB at SW10. (board specific).
 */

// Port Expander module connections
sbit  SPExpanderRST at LATF0_bit;
sbit  SPExpanderCS  at LATF1_bit;
sbit  SPExpanderRST_Direction at TRISF0_bit;
sbit  SPExpanderCS_Direction  at TRISF1_bit;
// End Port Expander module connections

unsigned int i = 0;

void main() {
  ADPCFG = 0xFFFF;                       // initialize AN pins as digital

  TRISB  = 0x00;
  LATB   = 0xFF;

  // If Port Expander Library uses SPI1 module
  SPI1_Init();                           // Initialize SPI module used with PortExpander

  Expander_Init(0);                      // Initialize Port Expander

  Expander_Set_DirectionPortA(0, 0x00);  // Set Expander's PORTA to be output

  Expander_Set_DirectionPortB(0,0xFF);   // Set Expander's PORTB to be input
  Expander_Set_PullUpsPortB(0,0xFF);     // Set pull-ups to all of the Expander's PORTB pins

  while(1) {                             // Endless loop
    Expander_Write_PortA(0, i++);        // Write i to expander's PORTA
    PORTB = Expander_Read_PortB(0);      // Read expander's PORTB and write it to LEDs
    Delay_ms(100);
  }

}