/*
 * Project name:
     RTC_Write (Demonstration on working with the RTC Module and Software I2C routines)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091030:
       - initial release;
 * Description:
      This project is simple demonstration how to set date and time on PCF8583
      RTC (real-time clock).
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC39
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL 8x, 10.00000 MHz
     Ext. Modules:    mE ac:RTC extra board on PORTF (board specific)
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
    - Pull-up I2C communication lines (RF3 and RF4)
       place jumper J5 in upper position. (board specific)
    - Turn off PORT LEDs at SW10. (board specific)
    - In order to use the example, address pin A0 of PCF8583 must be set to 0V.
       (on mikroElektronika's RTC module this is done by default)
 */

// Software I2C connections
sbit Soft_I2C_Scl           at RF3_bit;
sbit Soft_I2C_Sda           at RF4_bit;
sbit Soft_I2C_Scl_Direction at TRISF3_bit;
sbit Soft_I2C_Sda_Direction at TRISF4_bit;
// End Software I2C connections

void main() {
  ADPCFG = 0xFFFF;       // Initialize AN pins as digital

  Soft_I2C_Init();       // Initialize full master mode
  Soft_I2C_Start();      // Issue start signal
  Soft_I2C_Write(0xA0);  // Address PCF8583, see PCF8583 datasheet
  Soft_I2C_Write(0);     // Start from address 0 (configuration memory location)
  Soft_I2C_Write(0x80);  // Write 0x80 to configuration memory location (pause counter...)
  Soft_I2C_Write(0);     // Write 0 to cents memory location
  Soft_I2C_Write(0);     // Write 0 to seconds memory location
  Soft_I2C_Write(0x30);  // Write 0x30 to minutes memory location
  Soft_I2C_Write(0x12);  // Write 0x12 to hours memory location
  Soft_I2C_Write(0x30);  // Write 0x30 to year/date memory location
  Soft_I2C_Write(0x10);  // Write 0x10 to weekday/month memory location
  Soft_I2C_Stop();       // Issue stop signal

  Soft_I2C_Start();      // Issue start signal
  Soft_I2C_Write(0xA0);  // Address PCF8530
  Soft_I2C_Write(0);     // Start from address 0
  Soft_I2C_Write(0);     // Write 0 to configuration memory location (enable counting)
  Soft_I2C_Stop();       // Issue stop signal
}