/*
 * Project name:
     Spi_Lcd (Simple demonstration of the SPI Lcd 4-bit Library functions)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - initial release;
 * Description:
     This is a simple demonstration of SPI Lcd 4-bit library functions.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    ac:SPI_LCD adapter on PORTF, LCD 2x16
                      SmartADAPT Extra Development Board.
                      http://www.mikroe.com/add-on-boards/various/smartadapt/
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Use SmartADAPT to match the SPI pinout of the dsPIC30F4013 and Serial
         Lcd/Glcd adapter. (board specific)
 */

char *text = "mikroElektronika";

// Port Expander module connections
sbit SPExpanderRST at LATF0_bit;
sbit SPExpanderCS  at LATF1_bit;
sbit SPExpanderRST_Direction at TRISF0_bit;
sbit SPExpanderCS_Direction  at TRISF1_bit;
// End Port Expander module connections

char i;                              // Loop variable

void Move_Delay() {                  // Function used for text moving
  Delay_ms(500);                     // You can change the moving speed here
}

void main() {

  ADPCFG = 0xFFFF;                       // initialize AN pins as digital

  // If Port Expander Library uses SPI1 module
  SPI1_Init();                           // Initialize SPI module used with PortExpander

  SPI_Lcd_Config(0);                     // Initialize Lcd over SPI interface
  SPI_Lcd_Cmd(_LCD_CLEAR);               // Clear display
  SPI_Lcd_Cmd(_LCD_CURSOR_OFF);          // Turn cursor off
  SPI_Lcd_Out(1,6, "mikroE");            // Print text to Lcd, 1st row, 6th column
  SPI_Lcd_Chr_CP('!');                   // Append '!'
  SPI_Lcd_Out(2,1, text);                // Print text to Lcd, 2nd row, 1st column

  // SPI_Lcd_Out(3,1,"mikroE");          // For Lcd with more than two rows
  // SPI_Lcd_Out(4,15,"mikroE");         // For Lcd with more than two rows

  Delay_ms(2000);

  // Moving text
  for(i=0; i<4; i++) {                   // Move text to the right 4 times
    SPI_Lcd_Cmd(_LCD_SHIFT_RIGHT);
    Move_Delay();
  }

  while(1) {                             // Endless loop
    for(i=0; i<8; i++) {                 // Move text to the left 7 times
      SPI_Lcd_Cmd(_LCD_SHIFT_LEFT);
      Move_Delay();
    }

    for(i=0; i<8; i++) {                 // Move text to the right 7 times
      SPI_Lcd_Cmd(_LCD_SHIFT_RIGHT);
      Move_Delay();
    }
  }

}