/* Project name:
     WatchDog_Timer (Demonstration of the Watchdog Timer usage)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091106:
       - Initial release;
     20110511(TL):
       - Adapted for PIC24EP512GU810
 * Description:
     This code demonstrates how to embed Assembly language blocks
     and how to use hardware controlled Watchdog timer.
 * Test configuration:
     MCU:             PIC24EP512GU810
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70616g.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.00000 MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
   - Turn on Watchdog Timer in Edit Project window.
   - Turn on PORTB LEDs at SW15 (board specific).
   - This project is set to 1:128 WDT Prescaler and 1:256 WDT Postscaler.
 */

void main() {
  PLLPRE_0_bit = 0;
  PLLPRE_1_bit = 0;
  PLLPRE_2_bit = 0;
  PLLPRE_3_bit = 0;
  PLLPRE_4_bit = 0;

  PLLFBD = 38;             // PLL multiplier M=38

  PLLPOST_0_bit = 0;
  PLLPOST_1_bit = 0;

  ANSELA = 0x00;    // Convert all I/O pins to digital
  ANSELB = 0x00;
  ANSELC = 0x00;
  ANSELD = 0x00;
  ANSELE = 0x00;
  ANSELG = 0x00;


  asm CLRWDT;                 // asm line, clear WatchDog Timer
  LATB = 0x00FF;              // initialize PORTB
  TRISB = 0;                  // configure PORTB as output
  Delay_ms(300);              // wait 0.3 seconds
  LATB = 0xFF00;              // change PORTB value

  while (1)                   // endless loop, WatchDog_Timer will reset dsPIC
    ;
}