/*
 * Project name:
     Pwm_MC_Test (Demo on using mikroE's PWM Motor Control library functions)
 * Copyright:
     (c) MikroElektronika, 2012
 * Revision History:
     20091030:
       - Initial Release;
 * Description:
     This is a simple demonstration of PWM Motor Control library, which is being used for
     control of the dsPIC's motor control PWM module. The module is initialized and started,
     after which the motor control PWM duty ratio can be adjusted by means of 2 buttons
     connected to pins RB0 and RB1. The changes can be monitored on the PWM
     output pin RE0.
 * Test configuration:
     MCU:             dsPIC30F3010
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70141e.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
   - Turn off all PORT LEDs except PORTE at SW10. (board specific)
 */

unsigned int current_duty, pwm_period;

void main(){

  ADPCFG = 0xFFFF;                              // Initialize AN pins as digital
  PORTB  = 0;
  TRISB  = 0xFFFF;                              // Initialize portb as input
  current_duty = 10;
  Delay_ms(1000);

  pwm_period = PWM1_MC_Init(5000, 1, 0x01, 0);  // PWM1_MC_Init returns calculated timer period.
  PWM1_MC_Set_Duty(current_duty, 1);
  PWM1_MC_Start();

  while (1) {                                   // Endless loop
    if (RB0_bit) {                              // Button on RB0 pressed
      Delay_ms(20);
      
      current_duty++;                           // Increment current_duty
      if (current_duty > pwm_period) {          // If we increase current_duty greater then possible pwm_period value
        current_duty = 0;                       // Reset current_duty value to zero
      }
      PWM1_MC_Set_Duty(current_duty,  1);       // Set newly acquired duty ratio
     }

    if (RB1_bit) {                              // Button on RB1 pressed
      Delay_ms(20);
      current_duty--;                           // Decrement current_duty
      if (current_duty > pwm_period) {          // If we decrease current_duty greater then possible pwm_period value (overflow)
        current_duty = pwm_period;              // Set current_duty to max possible value
      }
      PWM1_MC_Set_Duty(current_duty,  1);       // Set newly acquired duty ratio
     }
    Delay_ms(5);                                // Slow down change pace a little
  }
}