/*
 * Project name:
     Eeprom_Test (Example on using the on-chip EEPROM library functions)
 * Target Platform:
      dsPIC;
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - initial release;
 * Description:
     This project demonstrates usage of EEPROM library functions. Each EEPROM
     (16-bit) location can be written to individually, or in 16-word blocks,
     which is somewhat faster than the former. If Writing in blocks, EEPROM data
     start address must be a multiply of 16. Please read mikroC's Help for more
     details on the library functions!
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn on PORTB LEDs switch SW10.1.
*/

unsigned int eeData;
unsigned long eeAddr;
unsigned int dArr[16];

void main() {

  unsigned i;

  ADPCFG = 0xFFFF;                         // Disable analog inputs
  
  TRISB = 0;                               // PORTB as output
  LATB = 0xFFFF;
  eeAddr = 0x7FFC00;                       // Start address of EEPROM
  eeData = 0;                              // Data to be written

  while (eeData <= 0x00FF) {
    EEPROM_Write(eeAddr, eeData++);        // Write data into EEPROM
    while(WR_bit);                         // Wait for write to finish,
    LATB = EEPROM_Read(eeAddr);            // then, read the just-written
                                           // data.
    eeAddr += 2;                           // Next address of EEPROM memory location

    Delay_ms(100);
  }

  Delay_ms(1000);                          // Wait 1 second.
  
  eeData = 0xAAAA;
  for (i=0; i<16; i++){                    // Initializing array of 16 integers with data
    dArr[i] = eeData;
    eeData = ~eeData;
  }

  EEPROM_Write_Block(0x7FFC20, dArr);      // Write entire row of EEPROM data
  while(WR_bit);                           // Wait for write to finish

  eeAddr = 0x7FFC20;                       // Address of EEPROM where reading should start
  for (i=0; i<16; i++){                    // Read the data back
    LATB = EEPROM_Read(eeAddr);            // and show it on PORTB
    eeAddr += 2;                           // Next address of EEPROM memory location
    Delay_ms(500);
  }
}