/* Project name:
      External Interrupt on PORTB (Simple 'Hello World' project)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20101007:
     - Initial release;
     20110224 (TL):
     - adapted for PRO version
 * Description:
     This is a simple 'Hello World' project. It detects press on PORTB.0 and
     shows result on diodes connected to PORTD.
 * Test configuration:
     MCU:          dsPIC30F4013
                   http://ww1.microchip.com/downloads/en/DeviceDoc/70138G.pdf
     Dev.Board:    EASYPIC v7 for dsPIC30
                   http://www.mikroe.com/easypic-dspic30/
     Oscillator:   XT-PLL, 10.000MHz
     Ext. Modules: None.
     SW:           mikroC PRO for dsPIC v4.60
                   http://www.mikroe.com/mikroc/dspic/

 * NOTES:
     - Make sure you turn ON the PORTB LEDs (SW10.1), set Keypad Pull Up Jumper in Vcc position (J15) (board specific)
       and turn on PORTA.11 Switch (SW8.7)
 */

int tmp = 0;                                          // Global variable tmp

void Ext_interrupt() org IVT_ADDR_INT0INTERRUPT {     // Interrupt rutine
  tmp = 1;                                            // Increment variable cnt
  IFS0.INT0IF = 0;                                    // Clear Interrupt Flag
}

void main() {                                         // Main program

  TRISB = 0;                                          // Set PortB as output
  LATB = 0;                                           // Starting value for PortB

  INTCON2 = 0x0001;                                   // Setup INT0 pin to interupt on falling edge
  IFS0.INT0IF = 0;                                    // Reset INT0 interrupt flag
  IEC0.INT0IE = 1;                                    // Enable INT0 Interrupt Service Routine

  TRISA.B11 = 1;                                      // Set PortA.11 as input

  while(1) {                                          // Unending loop
    while (tmp) {
      INTCON2 = 0x0000;                               // Disable interrupts
      LATB = 0xFF;                                    // Turn on PortB
      Delay_ms(5000);                                 // Wait for 5 seconds
      LATB = 0x00;                                    // Turn off PortB
      tmp = 0;                                        // Clear tmp variable
      INTCON2 = 0x0001;                               // Enable interrupts
    }
  }
}