/*
 * Project name:
     Flash_Test (on-chip Flash memory read/write example)
 * Target Platform:
     dsPIC;
 * Copyright:
     (c) mikroElektronika, 2012.
 * Revision History:
     20091106:
       - Initial Release;
 * Description:
     In this project, various read/write tecniques to/from the on-chip FLASH
     memory are shown.
     Flash memory is mapped to address space 3:2, meaning every 3 consecutive
     bytes of Flash have 2 consecutive address locations available.
     That is why mikroE's library allows data to be written to flash in two ways:
     'regular' and 'compact'. In 'regular' mode, which is used for variables that
     are size of 2 bytes and more, the 3rd (un-addressable) byte remains unused.
     In 'compact' mode, which can be used for 1 byte-sized variables/arrays, all
     bytes of flash are being used. For more information, please consult mikroC's
     Help.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Connect UART jumpers J20 and J21 to RS232.
     - Set RF2 at SW8 and RF3 at SW9 (board specific).
 */

unsigned int WriteWordArr[8] = {'*','m', 'i', 'k', 'r', 'o', 'E', '*'};
char WriteByteArr[] = "mikroElektronika Flash example";
char ReadByteArr[40];
unsigned int ReadWordArr[20];

void *ptr;

void main() {
  unsigned i;

  //--- erase the block first
  FLASH_Erase32(0x006000);

  //--- write compact format to flash
  FLASH_Write_Compact(0x006000, WriteByteArr, 36);
  /*
  This is what FLASH_Write_Compact() does 'beneath the hood'
 *
  FLASH_Write_Init(0x006000, pv1);
  FLASH_Write_Loadlatch4_Compact();
  FLASH_Write_Loadlatch4_Compact();
  FLASH_Write_Loadlatch4_Compact();
  FLASH_Write_DoWrite();
  */


  //--- read compact format
  ptr = ReadByteArr;
  FLASH_Read4_Compact(0x006000, ptr);
  ptr += 12;
  FLASH_Read4_Compact(0x006008, ptr);
  ptr += 12;
  FLASH_Read4_Compact(0x006010, ptr);
  ptr += 12;
  *ptr = 0;  //termination
  

  UART1_Init(9600);
  UART1_Write(13);
  UART1_Write(10);
  UART1_Write_Text("Start");
  UART1_Write(13);
  UART1_Write(10);
  //--- show what has been written
  i = 0;
  while(ReadByteArr[i]) {
    UART1_Write(ReadByteArr[i++]);
  }

  //--- now for some non-compact flash-write
  //--- erase the block first
  FLASH_Erase32(0x006100);
  FLASH_Write_Init(0x006100, WriteWordArr);
  FLASH_Write_Loadlatch4();
  FLASH_Write_Loadlatch4();
  FLASH_Write_DoWrite();
  
  //--- read non-compact format
  ptr = &ReadWordArr;
  FLASH_Read4(0x006100, ptr);
  ptr = ptr + 8;
  FLASH_Read4(0x006108, ptr);
  ptr = ptr + 8;
  *(unsigned *)ptr = 0;  //termination

  //--- show what has been written
  UART1_Write(13);
  UART1_Write(10);
  i = 0;
  while(ReadWordArr[i]){
      UART1_Write(ReadWordArr[i++]);
  }

}