/*
 * Project name:
     Pwm_Demo (Demo on using mikroE's PWM library functions)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091030:
       - Initial Release;
 * Description:
     This is a simple demonstration of PWM library, which is being used for
     control of the dsPIC's PWM module. The module is initialized and started,
     after which the PWM1 ans PWM2 Duty Ratios can be adjusted by means of 4 buttons
     connected to pins RB0, RB1, RB2 and RB3. The changes can be monitored on the PWM
     output pins (RD0 and RD1).
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
   - Turn off all PORT LEDs except PORTD at SW10. (board specific)
   - Pull-down PORTB and put jumper J15 in VCC position (board specific)
 */

unsigned int current_duty, old_duty, current_duty1, old_duty1;
unsigned int pwm_period1, pwm_period2;

void InitMain() {
  ADPCFG = 0xFFFF;                           // Configure AN pins as digital I/O
  TRISB = 0xFFFF;                            // configure PORTB pins as input
  PORTD = 0;                                 // set PORTD to 0
  TRISD = 0;                                 // designate PORTD pins as output
}

void main() {
  InitMain();
  current_duty  = 16;                        // initial value for current_duty
  current_duty1 = 16;                        // initial value for current_duty1

  pwm_period1 = PWM_Init(5000 , 1, 1, 2);
  pwm_period2 = PWM_Init(10000, 2, 1, 3);

  PWM_Start(1);
  PWM_Start(2);

  PWM_Set_Duty(current_duty,  1);            // Set current duty for PWM1
  PWM_Set_Duty(current_duty1, 2);            // Set current duty for PWM2

  while (1) {                                // endless loop
    if (RB0_bit) {                           // button on RB0 pressed
      Delay_ms(20);
      current_duty++;                        // increment current_duty
      if (current_duty > pwm_period1) {      // if we increase current_duty greater then possible pwm_period1 value
        current_duty = 0;                    // reset current_duty value to zero
      }
      PWM_Set_Duty(current_duty,  1);        // set newly acquired duty ratio
     }

    if (RB1_bit) {                           // button on RB1 pressed
      Delay_ms(20);
      current_duty--;                        // decrement current_duty
      if (current_duty > pwm_period1) {      // if we decrease current_duty greater then possible pwm_period1 value (overflow)
        current_duty = pwm_period1;          // set current_duty to max possible value
      }
      PWM_Set_Duty(current_duty,  1);        // set newly acquired duty ratio
     }

    if (RB2_bit) {                           // button on RB2 pressed
      Delay_ms(20);
      current_duty1++;                       // increment current_duty1
      if (current_duty1 > pwm_period2) {     // if we increase current_duty1 greater then possible pwm_period2 value
        current_duty1 = 0;                   // reset current_duty1 value to zero
      }
      PWM_Set_Duty(current_duty1,  2);       // set newly acquired duty ratio
     }

    if (RB3_bit) {                           // button on RB3 pressed
      Delay_ms(20);
      current_duty1--;                       // decrement current_duty1
      if (current_duty1 > pwm_period2) {     // if we decrease current_duty1 greater then possible pwm_period1 value (overflow)
        current_duty1 = pwm_period2;         // set current_duty to max possible value
      }
      PWM_Set_Duty(current_duty1,  2);
     }

    Delay_ms(5);                             // slow down change pace a little
  }
}