        /*
 * Project name:
     Timer1_interrupt (Timer1 interrupt test)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - Initial release;
 * Description:
     This code demonstrates how to use Timer1 and it's interrupt.
     Program toggles LEDs on PORTB.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Use the ORG directive to declare your procedure as an interrupt service routine.
       The address after the ORG statement indicates which interrupt you will assign to
       your procedure.
       Please consult the datasheet for your MCU for interrupt vector table
       details and available addresses.
     - Also, consult mikroC PRO for dsPIC30/33 and PIC24 help on how to use interrupts.
 */

void Timer1Int() iv IVT_ADDR_T1INTERRUPT {
  T1IF_bit = 0;             // Clear T1IF
  LATB = ~ PORTB;           // Invert PORTB
}

void main() {

  ADPCFG = 0xFFFF;          // Initialize AN pins as digital
  TRISB  = 0;               // initialize PORTB as output
  LATB   = 0xAAAA;          // Initialize PORTB value
  IPC0   = IPC0 | 0x1000;   // Interrupt priority level = 1
  T1IF_bit = 0;             // Clear T1IF
  T1IE_bit = 1;             // Enable Timer1 interrupts

  T1CON = 0x8030;           // Timer1 ON, internal clock FCY, prescaler 1:256

  while (1)                 // Endless loop, interrupted by Timer1Int
    ;
}