/*
 * Project name:
     Timer2_interrupt (Timer2 interrupt test)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091106:
       - Initial release;
 * Description:
     This code demonstrates how to use Timer2 and it's interrupt.
     Program toggles LEDs on PORTB.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Use the ORG directive to declare your procedure as an interrupt service routine.
       The address after the ORG statement indicates which interrupt you will assign to
       your procedure.
       Please consult the datasheet for your MCU for interrupt vector table
       details and available addresses.
     - Also, consult mikroC PRO for dsPIC30/33 and PIC24 help on how to use interrupts.
 */

void Timer2Int() iv IVT_ADDR_T2INTERRUPT {
  T2IF_bit = 0;             // Clear T2IF
  LATB = ~ PORTB;           // Invert PORTB
}

void main() {

  ADPCFG = 0xFFFF;          // Initialize AN pins as digital
  TRISB  = 0;               // Initialize PORTB as output
  LATB   = 0xAAAA;          // Initialize PORTB value
  IPC0   = IPC0 | 0x1000;   // Interrupt priority level = 1
  T2IF_bit = 0;             // Clear T2IF
  T2IE_bit = 1;             // Enable Timer2 interrupts

  T2CON = 0x8030;           // Timer2 ON, internal clock FCY, prescaler 1:256

  while (1)                 // Endless loop, interrupted by Timer2Int
    ;
}