/* Project name:
     Timer3_interrupt (Timer3 interrupt test)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091106:
       - Initial release;
     20110511(TL):
       - Adapted for dsPIC33EP512MU810
 * Description:
     This code demonstrates how to use Timer3 and it's interrupt.
     Program toggles LEDs on PORTB.
 * Test configuration:
     MCU:             dsPIC33EP512MU810
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70616g.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      HS-PLL, 80.00000 MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
                      Timer Calculator
                      http://www.libstock.com/projects/view/398/timer-calculator
 * NOTES:
   - Use the ORG directive to declare your procedure as an interrupt service routine.
     The address after the ORG statement indicates which interrupt you will assign to
     your procedure.
     Please consult the datasheet for your MCU for interrupt vector table
     details and available addresses.
   - Also, consult mikroC PRO for dsPIC30/33 and PIC24 help on how to use interrupts.
   - Turn off all PORT LEDs except PORTB at SW15 (board specific).
 */

void Timer3Int() iv IVT_ADDR_T3INTERRUPT {
  T3IF_bit = 0;               // clear T3IF
  LATB = ~ PORTB;             // invert PORTB
}

void main() {
  PLLPRE_0_bit = 0;
  PLLPRE_1_bit = 0;
  PLLPRE_2_bit = 0;
  PLLPRE_3_bit = 0;
  PLLPRE_4_bit = 0;

  PLLFBD = 38;             // PLL multiplier M=38

  PLLPOST_0_bit = 0;
  PLLPOST_1_bit = 0;

  ANSELA = 0x00;    // Convert all I/O pins to digital
  ANSELB = 0x00;
  ANSELC = 0x00;
  ANSELD = 0x00;
  ANSELE = 0x00;
  ANSELG = 0x00;

  TRISB  = 0;                 // initialize PORTB as output
  LATB   = 0xAAAA;            // initialize PORTB value
  IPC0   = IPC0 | 0x1000;     // interrupt priority level = 1
  T3IF_bit = 0;               // clear T3IF
  T3IE_bit = 1;               // enable Timer3 interrupts

  T3CON = 0x8020;             // Timer3 ON, prescaler 1:64
  PR2   = 62500;            // 100ms interrupt

  while (1)                   // endless loop, interrupted by Timer3Int
    ;
}