/* Project name:
     WatchDog_Timer (Demonstration of the Watchdog Timer usage)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091106:
       - Initial release;
     20110511(TL):
       - Adapted for dsPIC33FJ256GP710A
 * Description:
     This code demonstrates how to embed Assembly language blocks
     and how to use hardware controlled Watchdog timer.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593C.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      HS-PLL, 40.00000 MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
   - Turn on Watchdog Timer in Edit Project window.
   - Turn on PORTB LEDs at SW15 (board specific).
   - This project is set to 1:128 WDT Prescaler and 1:256 WDT Postscaler.
 */

void main() {
  PLLFBD = 0x1E;              // with 10 MHz input
                              // PLLFBD must be set to 0x1E  to make 40 MHz output
                              // see family reference for details
  AD1PCFGL = 0xFFFF;          // initialize AN pins as digital
  AD1PCFGH = 0xFFFF;          // initialize AN pins as digital
  AD2PCFGL = 0xFFFF;          // initialize AN pins as digital


  asm CLRWDT;                 // asm line, clear WatchDog Timer
  LATB = 0x00FF;              // initialize PORTB
  TRISB = 0;                  // configure PORTB as output
  Delay_ms(300);              // wait 0.3 seconds
  LATB = 0xFF00;              // change PORTB value

  while (1)                   // endless loop, WatchDog_Timer will reset dsPIC
    ;
}