/* Project name:
     Flash_Test (on-chip Flash memory read/write example)
 * Copyright:
     (c) mikroElektronika, 2012.
 * Revision History:
     20091106:
       - Initial Release;
     20110510(TL):
       - adapted for dsPIC33FJ256GP710A;
 * Description:
     In this project, various read/write tecniques to/from the on-chip FLASH
     memory are shown.
     Flash memory is mapped to address space 3:2, meaning every 3 consecutive
     bytes of Flash have 2 consecutive address locations available.
     That is why mikroE's library allows data to be written to flash in two ways:
     'regular' and 'compact'. In 'regular' mode, which is used for variables that
     are size of 2 bytes and more, the 3rd (un-addressable) byte remains unused.
     In 'compact' mode, which can be used for 1 byte-sized variables/arrays, all
     bytes of flash are being used. For more information, please consult mikroC's
     Help.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593C.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.00000 MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn ON switches for UART A at SW12 (board specific).
     - Pull-down PORTB.B0 switch at PORTB/L (board specific).
     - Set Button Press Level high at SW10 (board specific).
 */
unsigned int WriteWordArr[64] = {'m', 'i', 'k', 'r', 'o', 'E', 'l', 'e', 'k', 't', 'r', 'o', 'n', 'i', 'k', 'a'};
char         WriteByteArr[192] = "Flash example made by mikroElektronika";
unsigned int ReadWordArr[64];
char ReadByteArr[192];

void main() {
  unsigned i, k;

  // PLL settings
  PLLPRE_4_bit = 0;                  // PLLPRE<4:0> = 0  ->  N1 = 2    8MHz / 2 = 4MHz
  PLLPRE_3_bit = 0;                  // (must be within 0.8 MHz to 8 MHz range)
  PLLPRE_2_bit = 0;
  PLLPRE_1_bit = 0;
  PLLPRE_0_bit = 0;

  PLLFBD =   38;                     // PLLDIV<8:0> = 38 ->  M = 40    4MHz * 40 = 160MHz
                                     // (must be within 100 MHz to 200 MHz range)
  PLLPOST_0_bit = 0;                 // PLLPOST<1:0> = 0 ->  N2 = 2    160MHz / 2 = 80MHz
  PLLPOST_0_bit = 0;                 // (must be within 12.5 MHz to 80 MHz range)
  
  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;
  AD2PCFGL = 0xFFFF;

  for(i=16;i<64;i++)       // fill the rest of writeWordArr with 0
    WriteWordArr[i] = 0;

  for(i=60;i<192;i++)      // fill the rest of writeWordArr with 0
    WriteByteArr[i] = 0;

  UART2_Init(9600);
  UART2_Write_Text("Press RB0 to continue...");

  // press RB0 to continue
  while (!RB0_bit)
    ;
  i = 0;
  UART2_Write(13);
  UART2_Write(10);
  UART2_Write_Text("Start");
  UART2_Write(13);
  UART2_Write(10);
  // --- erase the block first
  FLASH_Erase(0x006400);

  // --- now for some non-compact flash-write
  FLASH_Write(0x006500, WriteWordArr);

  // --- read compact format
  FLASH_Read(0x006500, ReadWordArr, 64);
  // --- show what has been written
  i = 0;
  while(ReadWordArr[i])
    UART2_Write(ReadWordArr[i++]);
  UART2_Write(13);
  UART2_Write(10);
  //--- write compact format to flash
  FLASH_Write_Compact(0x006400, WriteByteArr);

  //--- read compact format
  FLASH_Read_Compact(0x006400, ReadByteArr, 192);

  //--- show what has been written
  i = 0 ;
  while(ReadByteArr[i])
    UART2_Write(ReadByteArr[i++]);
    UART2_Write(13);
    UART2_Write(10);
    UART2_Write_Text("End");
}