/* Project name:
     PWM (PWM library Demonstration)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091030:
       - Initial release;
     20110511(TL):
       - Adapted for dsPIC33FJ256GP710A
 * Description:
     This is a simple demonstration of PWM library, which is being used for
     control of the dsPIC's PWM module. The module is initialized and started,
     after which the PWM1 ans PWM2 Duty Ratios can be adjusted by means of 4 buttons
     connected to pins RA0, RA1, RA2 and RA3. The changes can be monitored on the PWM
     output pins (RD0 and RD1).
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593C.pdf
     Dev.Board:       EasyPIC Fusion v7
'                     http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.00000 MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
   - Turn off all PORT LEDs except PORTD at SW15 (board specific).
   - Pull-up PORTA at Button Press Level, put-down RA0, RA1, RA2 and RA3 and put 
     jumper J1 in VCC position (board specific).
 */

unsigned int current_duty, old_duty, current_duty1, old_duty1;
unsigned int pwm_period1, pwm_period2;

void InitMain() {
    // PLL settings
  CLKDIVbits.PLLPRE = 0;      // PLLPRE<4:0> = 0  ->  N1 = 2    8MHz / 2 = 4MHz
                              // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD =   38;              // PLLDIV<8:0> = 38 ->  M = 40    4MHz * 40 = 160MHz
                              // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0;     // PLLPOST<1:0> = 0 ->  N2 = 2    160MHz / 2 = 80MHz
                              // (must be within 12.5 MHz to 80 MHz range)

  ADPCFG = 0xFFFF;                           // configure AN pins as digital I/O
  TRISA = 0xFFFF;                            // configure PORTA pins as input
  PORTD = 0;                                 // set PORTD to 0
  TRISD = 0;                                 // designate PORTD pins as output
}

void main() {
  InitMain();
  current_duty  = 16;                        // initial value for current_duty
  current_duty1 = 16;                        // initial value for current_duty1

  pwm_period1 = PWM_Init(5000 , 1, 1, 2);
  pwm_period2 = PWM_Init(10000, 2, 1, 3);

  PWM_Start(1);
  PWM_Start(2);

  PWM_Set_Duty(current_duty,  1);            // set current duty for PWM1
  PWM_Set_Duty(current_duty1, 2);            // set current duty for PWM2

  while (1) {                                // endless loop
    if (RA0_bit) {                           // button on RA0 pressed
      Delay_ms(20);
      current_duty++;                        // increment current_duty
      if (current_duty > pwm_period1) {      // if we increase current_duty greater then possible pwm_period1 value
        current_duty = 0;                    // reset current_duty value to zero
      }
      PWM_Set_Duty(current_duty,  1);        // set newly acquired duty ratio
     }

    if (RA1_bit) {                           // button on RA1 pressed
      Delay_ms(20);
      current_duty--;                        // decrement current_duty
      if (current_duty > pwm_period1) {      // if we decrease current_duty greater then possible pwm_period1 value (overflow)
        current_duty = pwm_period1;          // set current_duty to max possible value
      }
      PWM_Set_Duty(current_duty,  1);        // set newly acquired duty ratio
     }

    if (RA2_bit) {                           // button on RA2 pressed
      Delay_ms(20);
      current_duty1++;                       // increment current_duty1
      if (current_duty1 > pwm_period2) {     // if we increase current_duty1 greater then possible pwm_period2 value
        current_duty1 = 0;                   // reset current_duty1 value to zero
      }
      PWM_Set_Duty(current_duty1,  2);       // set newly acquired duty ratio
     }

    if (RA3_bit) {                           // button on RA3 pressed
      Delay_ms(20);
      current_duty1--;                       // decrement current_duty1
      if (current_duty1 > pwm_period2) {     // if we decrease current_duty1 greater then possible pwm_period1 value (overflow)
        current_duty1 = pwm_period2;         // set current_duty to max possible value
      }
      PWM_Set_Duty(current_duty1,  2);
     }

    Delay_ms(5);                             // slow down change pace a little
  }
}