/* Project name:
     Timer2_interrupt (Timer2 interrupt test)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091106:
       - Initial release;
     20110511(TL):
       - adapted for dsPIC33FJ256GP710A
 * Description:
     This code demonstrates how to use Timer2 and it's interrupt.
     Program toggles LEDs on PORTB.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593C.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.00000 MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
                      Timer Calculator
                      http://www.libstock.com/projects/view/398/timer-calculator
 * NOTES:
   - Use the ORG directive to declare your procedure as an interrupt service routine.
     The address after the ORG statement indicates which interrupt you will assign to
     your procedure.
     Please consult the datasheet for your MCU for interrupt vector table
     details and available addresses.
   - Also, consult mikroC PRO for dsPIC30/33 and PIC24 help on how to use interrupts.
   - Turn off all PORT LEDs except PORTB at SW15 (board specific).
 */

void Timer2Int() iv IVT_ADDR_T2INTERRUPT {
  T2IF_bit = 0;             // clear T2IF
  LATB = ~ PORTB;           // invert PORTB
}

void main() {
    // PLL settings
  CLKDIVbits.PLLPRE = 0;      // PLLPRE<4:0> = 0  ->  N1 = 2    8MHz / 2 = 4MHz
                              // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD =   38;              // PLLDIV<8:0> = 38 ->  M = 40    4MHz * 40 = 160MHz
                              // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0;     // PLLPOST<1:0> = 0 ->  N2 = 2    160MHz / 2 = 80MHz
                              // (must be within 12.5 MHz to 80 MHz range)
                              
  AD1PCFGL = 0xFFFF;        // initialize AN pins as digital
  AD1PCFGH = 0xFFFF;        // initialize AN pins as digital
  AD2PCFGL = 0xFFFF;        // initialize AN pins as digital
  TRISB  = 0;               // initialize PORTB as output
  LATB   = 0xAAAA;          // initialize PORTB value
  IPC0   = IPC0 | 0x1000;   // interrupt priority level = 1
  T2IF_bit = 0;             // clear T2IF
  T2IE_bit = 1;             // enable Timer2 interrupts

  T2CON = 0x8020;           // Timer2 ON, internal clock FCY, prescaler 1:64
  PR2   = 62500;            // 100ms interrupt

  while (1)                 // endless loop, interrupted by Timer2Int
    ;
}