/*
 * Project name:
     PS2_Example (Demonstration on using PS/2 keyboard library)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - initial release;
 * Description:
     In this example, key(s) pressed on the PS/2 keyboard are read and transferred
     to PC through serial port connection. Various basic keyboard activities are
     tested: "normal" keys, keys with <Shift> pressed, keys with <Caps Lock>
     pressed, numerical keypad ON/OFF and keys. The result is visible on PC, on
     USART Terminal tool.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    Standard PS2 keyboard
                      http://en.wikipedia.org/wiki/Keyboard_(computing)
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Some keyboards with various multimedia attachments on them 
       tend to "choke" the communication by constantly sending
       requests on various multimedia objects status (volume, mouse pos. etc.).
       This may slow down the communication pace with the MCU.
     - Diodes off (SW10).
     - Connect UART jumpers J20 and J21 to RS232 and RF3 at SW9 and RF2 at SW8 (board specific).
 */


unsigned short keydata , special , down;
 
sbit PS2_Data            at RB0_bit;
sbit PS2_Clock           at RB1_bit;
sbit PS2_Data_Direction  at TRISB0_bit;
sbit PS2_Clock_Direction at TRISB1_bit;

void main() {

  ADPCFG = 0xFFFF;                                    // Configure AN pins as digital I/O
  
  UART1_Init(19200);                                  // Initialize UART module at 19200 bps


  Ps2_Config();                                       // Init PS/2 Keyboard
  Delay_ms(100);                                      // Wait for keyboard to finish
  UART1_Write_Text("Ready");
  UART1_Write(10);                                    // Line Feed
  UART1_Write(13);                                    // Carriage return

  do {
    if (Ps2_Key_Read(&keydata, &special, &down)) {
      if (down && (keydata == 16)) {                  // Backspace
         UART1_Write(0x08);
      }
      else if (down && (keydata == 13)) {             // Enter
        UART1_Write('\r');                            // send carriage return to usart terminal
        //UART1_Write('\n');                          // uncomment this line if usart terminal also expects line feed
                                                      // for new line transition
      }
      else if (down && !special && keydata) {
        UART1_Write(keydata);                         // Send key to usart terminal
      }
    }
    Delay_ms(1);                                      // Debounce period
  } while (1);
}