/*
 * Project name:
     SetjmpDemo (Example on using the Setjmp library)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - initial release;
 * Description:
     Example demonstrates function cross calling using setjmp and longjmp functions.
     When called, Setjmp() saves its calling environment in its jmp_buf argument
     for later use by the Longjmp().
     Longjmp(), on the other hand, restores the environment saved by the most
     recent invocation of the Setjmp() with the corresponding jmp_buf argument.

 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    None
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     -  Invocation of Longjmp must occur before return from the function in which
        Setjmp was called encounters.
 */

#include <Setjmp.h>

jmp_buf buf;             // Note: Program flow diagrams are indexed according
                         //       to the sequence of execution

void func33(){           //  2<------------|
  Delay_ms(1000);        //                |
                         //                |
  asm nop;               //                |
  Longjmp(buf, 2);       //  3---------------->|
  asm nop;               //                |   |
                         //                |   |
}                        //                |   |
                         //                |   |
void func(){             //  1<--------|   |   |
                         //            |   |   |
  PORTB = 3;             //            |   |   |
  if (Setjmp(buf) == 2)  //  3<----------------|
    PORTB = 1;           //  4-->|     |   |
  else                   //      |     |   |
    func33();            //  2------------>|
                         //      |     |
                         //  4<--|     |
}                        //  5----->|  |
                         //         |  |
void main() {            //         |  |
  ADPCFG = 0xFFFF;       //         |  |
                         //         |  |
  PORTB = 0;             //         |  |
  TRISB = 0;             //         |  |
                         //         |  |
  asm nop;               //         |  |
                         //         |  |
  func();                //  1-------->|
                         //         |
  asm nop;               //  5<-----|
  Delay_ms(1000);
  PORTB = 0xFFFF;

}